from distilabel.llm import LLM, ProcessLLM
from distilabel.pipeline import Pipeline
from distilabel.tasks import UltraFeedbackTask


def load_gpt_4(task: UltraFeedbackTask) -> LLM:
    from distilabel.llm import OpenAILLM

    return OpenAILLM(
        model="gpt-4-1106-preview",
        task=task,
        max_new_tokens=512,
        num_threads=4,
    )


pipeline = Pipeline(
    generator=pool,
    labeller=ProcessLLM(task=UltraFeedbackTask(), load_llm_fn=load_gpt_4),  # (1)
)



import os
from distilabel.llm import OpenAILLM
from distilabel.tasks import JudgeLMTask

llm = OpenAILLM(
    task=JudgeLMTask(),
    openai_api_key=os.getenv("OPENAI_API_KEY", None),
    temperature=0.3,
)
print(
    llm.validate_prompts(
        [
            {
                "input": "What's a large language model?",
                "generations": [
                    "A Large Language Model (LLM) is a type of artificial intelligence that processes and generates human-like text based on vast amounts of training data.",
                    "Sorry I cannot answer that."
                ]
            }
        ]
    )[0]
)
# You are a helpful and precise assistant for checking the quality of the answer.
# [Question]
# What's an LLM?


# [The Start of Assistant 1's Answer>
# A Large Language Model (LLM) is a type of artificial intelligence that processes and generates human-like text based on vast amounts of training data.
# [The End of Assistant 1's Answer>
# [The Start of Assistant 2's Answer>
# Sorry I cannot answer that.
# [The End of Assistant 2's Answer>

# [System]
# We would like to request your feedback on the performance of 2 AI assistants in response to the user question displayed above.
# Please rate the helpfulness, relevance, accuracy, level of details of their responses. Each assistant receives an overall score on a scale of 1 to 10, where a higher score indicates better overall performance.
# Please first output a single line containing only 2 values indicating the scores for Assistants 1 to 2, respectively. The 2 scores are separated by a space. In the subsequent line, please provide a comprehensive explanation of your evaluation, avoiding any potential bias and ensuring that the order in which the responses were presented does not affect your judgment.


# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
from dataclasses import dataclass
from typing import Any, Dict, List

from distilabel.tasks.base import get_template
from distilabel.tasks.preference.base import PreferenceTaskNoRationale
from distilabel.tasks.prompt import Prompt

_COMPLEXITY_SCORER_TEMPLATE = get_template("complexity-scorer.jinja2")


@dataclass
class ComplexityScorerTask(PreferenceTaskNoRationale):
    """A `PreferenceTask` following the `Complexity Scorer` specification for rating instructions
    in terms of complexity.

    This task is inspired by the Evol Complexity Scorer in the Deita framework: *Deita is an open-sourced project
    designed to facilitate Automatic Data Selection for instruction tuning in Large Language Models (LLMs).*

    The task is defined as follows:
    Ask an LLM (in the original paper they used ChatGPT) to rate the instructions (the number of instructions
    is dynamic in the sense that you can compare any number, in *Deita* the chose 6) to obtain a complexity
    score *c* for each instruction.

    This task will only need to receive the list of `generations` in a dataset to generate the scores.

    Args:
        system_prompt (str, optional): the system prompt to be used. Not defined for this task.

    References:
        - [`What Makes Good Data for Alignment? A Comprehensive Study of Automatic Data Selection in Instruction Tuning`](https://arxiv.org/abs/2312.15685)
    """

    system_prompt: str = ""

    __jinja2_template__: str = _COMPLEXITY_SCORER_TEMPLATE

    def generate_prompt(self, generations: List[str], **_: Any) -> Prompt:
        """Generates a prompt following the *Evol Complexity* specification in *Deita*.

        Args:
            generations (List[str]): the generations to be used for the prompt.

        Returns:
            Prompt: the generated prompt.

        Examples:
            >>> from distilabel.tasks import ComplexityScorerTask
            >>> task = ComplexityScorerTask()
            >>> task.generate_prompt(["instruction 1", "instruction 2"])
            Prompt(system_prompt="", formatted_prompt="Ranking the following questions...")
        """
        render_kwargs = {"instructions": generations}
        return Prompt(
            system_prompt=self.system_prompt,
            formatted_prompt=self.template.render(**render_kwargs),
        )

    @property
    def input_args_names(self) -> List[str]:
        """Returns the names of the input arguments of the task."""
        return ["generations"]

    def parse_output(self, output: str) -> Dict[str, List[str]]:
        """Parses the output of the task, returning a list with the rank/score of each instruction.

        Args:
            output (str): The output of the LLM raw.

        Returns:
            Dict[str, List[str]]: A dict with containing the ranks/scores of each instruction.
        """
        output = output.lower().split("\n")
        scores = [float(re.sub(r"\[\d+\] score:", "", o).strip()) for o in output]
        return {self.output_args_names[0]: scores}



import os

from distilabel.llm import OpenAILLM
from distilabel.tasks import UltraFeedbackTask

labeller = OpenAILLM(
    task=UltraFeedbackTask.for_helpfulness(),
    api_key=os.getenv("OPENAI_API_KEY", None),
)



from distilabel.tasks import UltraJudgeTask

# To see the complete system_prompt and task_description please take a look at the UltraJudgeTask definition
ultrajudge_task = UltraJudgeTask(
    system_prompt="You are an evaluator tasked with assessing AI assistants' responses from the perspective of typical user preferences...",
    task_description="Your task is to rigorously evaluate the performance of...",
    areas=[
        "Practical Accuracy",
        "Clarity & Transparency",
        "Authenticity & Reliability",
        "Compliance with Intent",
    ],
)



# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
from typing import TYPE_CHECKING, Any, Callable, Dict, Generator, List, Union
from urllib import error

from tenacity import (
    after_log,
    before_sleep_log,
    retry,
    retry_if_exception_type,
    stop_after_attempt,
    wait_random_exponential,
)

from distilabel.llm.base import LLM
from distilabel.llm.utils import LLMOutput
from distilabel.logger import get_logger
from distilabel.utils.imports import _OLLAMA_AVAILABLE

if _OLLAMA_AVAILABLE:
    import ollama

if TYPE_CHECKING:
    from distilabel.tasks.base import Task
    from distilabel.tasks.prompt import SupportedFormats

logger = get_logger()

_OLLAMA_API_RETRY_ON_EXCEPTIONS = (error.HTTPError,)
_OLLAMA_API_STOP_AFTER_ATTEMPT = 6
_OLLAMA_API_WAIT_RANDOM_EXPONENTIAL_MULTIPLIER = 1
_OLLAMA_API_WAIT_RANDOM_EXPONENTIAL_MAX = 10


class OllamaLLM(LLM):
    OLLAMA_HOST = os.environ.get("OLLAMA_HOST", "http://localhost:11434")

    def __init__(
        self,
        model: str,
        task: "Task",
        max_new_tokens: Union[int, None] = None,
        temperature: Union[float, None] = None,
        top_k: Union[int, None] = None,
        top_p: Union[float, None] = None,
        mirostat: Union[int, None] = None,
        mirostat_eta: Union[float, None] = None,
        mirostat_tau: Union[float, None] = None,
        num_ctx: Union[int, None] = None,
        num_gqa: Union[int, None] = None,
        num_gpu: Union[int, None] = None,
        num_threads: Union[int, None] = None,
        repeat_last_n: Union[int, None] = None,
        repeat_penalty: Union[float, None] = None,
        seed: Union[int, None] = None,
        stop: Union[str, None] = None,
        tfs_z: Union[float, None] = None,
        prompt_format: Union["SupportedFormats", None] = None,
        prompt_formatting_fn: Union[Callable[..., str], None] = None,
    ) -> None:
        """
        Initializes the OllamaLLM class and aligns with https://github.com/ollama/ollama/blob/main/docs/modelfile.md#valid-parameters-and-values

        Args:
            model (str): the model to be used for generation.
            task (Task): the task to be performed by the LLM.
            max_new_tokens (int, optional): the maximum number of tokens to be generated.
                Defaults to `None`.
            temperature (float, optional): the temperature to be used for generation.
                Defaults to `None`.
            top_k (int, optional): the top-k value to be used for generation.
                Defaults to `None`.
            top_p (float, optional): the top-p value to be used for generation.
                Defaults to `None`.
            mirostat (int, optional): the Mirostat value to enable it or set the version.
                Defaults to `None`.
            mirostat_eta (float, optional): the eta value to be used for Mirostat.
                Defaults to `None`.
            mirostat_tau (float, optional): the tau value to be used for Mirostat.
                Defaults to `None`.
            num_ctx (int, optional): the number of contexts to be used for generation.
                Defaults to `None`.
            num_gqa (int, optional): the number of GQA to be used for generation.
                Defaults to `None`.
            num_gpu (int, optional): the number of GPUs to be used for generation.
                Defaults to `None`.
            num_threads (Union[int, None], optional): the number of threads to be used
                for parallel generation. If `None`, no parallel generation will be performed.
                Defaults to `None`.
            repeat_last_n (Union[int, None], optional): the number of tokens to be used
                for RepeatLastN. Defaults to `None`.
            repeat_penalty (Union[float, None], optional): the penalty to be used for RepeatLastN.
                Defaults to `None`.
            seed (Union[int, None], optional): the seed to be used for generation.
                Defaults to `None`.
            stop (Union[str, None], optional): the stop token to be used for generation. If `None`,
                no stop token will be used. Defaults to `None`.
            tfs_z (Union[float, None], optional): the z value to be used for TFS.
                Defaults to `None`.
            prompt_format (Union[SupportedFormats, None], optional): the format to be used
                for the prompt. If `None`, the default format of the task will be used, available
                formats are `openai`, `chatml`, `llama2`, `zephyr`, and `default`. Defaults to `None`,
                but `default` (concatenation of `system_prompt` and `formatted_prompt` with a line-break)
                will be used if no `prompt_formatting_fn` is provided.
            prompt_formatting_fn (Union[Callable[..., str], None], optional): a function to be
                applied to the prompt before generation. If `None`, no formatting will be applied.
                Defaults to `None`..

        Raises:
            ValueError: if the model is not available.
            ValueError: if the Ollama API request failed.

        Examples:
            >>> from distilabel.tasks import TextGenerationTask
            >>> from distilabel.llm import OllamaLLM
            >>> llm = OllamaLLM(model="notus", task=TextGenerationTask())
            >>> llm.generate([{"input": "What's the capital of Spain?"}])
        """
        super().__init__(
            task=task,
            num_threads=num_threads,
            prompt_format=prompt_format,
            prompt_formatting_fn=prompt_formatting_fn,
        )

        self.model = model
        self.max_new_tokens = max_new_tokens
        self.temperature = temperature
        self.top_k = top_k
        self.top_p = top_p
        self.mirostat = mirostat
        self.mirostat_eta = mirostat_eta
        self.mirostat_tau = mirostat_tau
        self.num_ctx = num_ctx
        self.num_gqa = num_gqa
        self.num_gpu = num_gpu
        self.repeat_last_n = repeat_last_n
        self.repeat_penalty = repeat_penalty
        self.seed = seed
        self.stop = stop
        self.tfs_z = tfs_z

        self._api_available()
        self._api_model_available()

    @property
    def model_name(self) -> str:
        """Returns the name of the Ollama model."""
        return self.model

    def _api_available(self):
        """Checks if the Ollama API is available."""
        try:
            ollama.list()
        except ollama.ResponseError as e:
            raise ValueError(
                f"Could not connect to Ollama at {self.OLLAMA_HOST}. Check https://github.com/ollama/ollama-python/tree/main for deployment guide."
            ) from e

    def _api_model_available(self):
        """Checks if the Ollama model is available"""
        try:
            ollama.show(self.model)
        except ollama.ResponseError as e:
            raise ValueError(
                f"Model {self.model} is not available. Run `ollama run {self.model}` to serve the model."
            ) from e

    @retry(
        retry=retry_if_exception_type(_OLLAMA_API_RETRY_ON_EXCEPTIONS),
        stop=stop_after_attempt(_OLLAMA_API_STOP_AFTER_ATTEMPT),
        wait=wait_random_exponential(
            multiplier=_OLLAMA_API_WAIT_RANDOM_EXPONENTIAL_MULTIPLIER,
            max=_OLLAMA_API_WAIT_RANDOM_EXPONENTIAL_MAX,
        ),
        before_sleep=before_sleep_log(logger, logging.INFO),
        after=after_log(logger, logging.INFO),
    )
    def _text_generation_with_backoff(
        self, prompt: List[Dict[str, str]], **kwargs
    ) -> str:
        """Generates text using the Ollama API with backoff."""
        try:
            return ollama.chat(
                model=self.model,
                messages=prompt,
                options={
                    "num_predict": self.max_new_tokens,
                    "temperature": self.temperature,
                    "top_p": self.top_p,
                    "top_k": self.top_k,
                    "mirostat": self.mirostat,
                    "mirostat_eta": self.mirostat_eta,
                    "mirostat_tau": self.mirostat_tau,
                    "num_ctx": self.num_ctx,
                    "num_gqa": self.num_gqa,
                    "num_gpu": self.num_gpu,
                    "repeat_last_n": self.repeat_last_n,
                    "repeat_penalty": self.repeat_penalty,
                    "seed": self.seed,
                    "stop": self.stop,
                    "tfs_z": self.tfs_z,
                },
            )
        except ollama.ResponseError as e:
            if e.status_code >= 500:
                raise
            else:
                raise ValueError(
                    f"Ollama API request failed with status_code {e.status_code}."
                ) from e

    def __rich_repr__(self) -> Generator[Any, None, None]:
        yield from super().__rich_repr__()
        yield (
            "parameters",
            {
                "model": self.model,
                "max_new_tokens": self.max_new_tokens,
                "temperature": self.temperature,
                "top_k": self.top_k,
                "top_p": self.top_p,
                "mirostat": self.mirostat,
                "mirostat_eta": self.mirostat_eta,
                "mirostat_tau": self.mirostat_tau,
                "num_ctx": self.num_ctx,
                "num_gqa": self.num_gqa,
                "num_gpu": self.num_gpu,
                "repeat_last_n": self.repeat_last_n,
                "repeat_penalty": self.repeat_penalty,
                "seed": self.seed,
                "stop": self.stop,
                "tfs_z": self.tfs_z,
            },
        )

    def _generate(
        self, inputs: List[Dict[str, Any]], num_generations: int = 1
    ) -> List[List[LLMOutput]]:
        prompts = self._generate_prompts(inputs, default_format="openai")
        outputs = []
        for prompt in prompts:
            responses = [
                self._text_generation_with_backoff(prompt=prompt)
                for _ in range(num_generations)
            ]
            output = []
            for response in responses:
                raw_output = response.get("message", {}).get("content")  # type: ignore
                try:
                    parsed_response = self.task.parse_output(raw_output.strip())
                except Exception as e:
                    logger.error(f"Error parsing OpenAI response: {e}")
                    parsed_response = None
                output.append(
                    LLMOutput(
                        model_name=self.model_name,
                        prompt_used=prompt,
                        raw_output=raw_output,
                        parsed_output=parsed_response,
                    )
                )
            outputs.append(output)
        return outputs



# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os

from rich.logging import RichHandler

DISTILABEL_LOG_LEVEL = os.environ.get("DISTILABEL_LOG_LEVEL", "INFO").upper()


logging.basicConfig(
    level=logging.INFO,
    format="[PID: %(process)d] %(message)s",
    datefmt="%X",
    handlers=[RichHandler(rich_tracebacks=True, tracebacks_show_locals=True)],
)

# Silence httpx logs
logging.getLogger("httpx").setLevel(logging.WARNING)


def _get_root_logger() -> logging.Logger:
    return logging.getLogger("distilabel")


def _configure_root_logger():
    root_logger = _get_root_logger()
    root_logger.setLevel(DISTILABEL_LOG_LEVEL)


_configure_root_logger()


def get_logger() -> logging.Logger:
    return logging.getLogger("distilabel")



import os

from distilabel.llm import OpenAILLM
from distilabel.tasks import ComplexityScorerTask

labeller = OpenAILLM(
    task=ComplexityScorerTask(),
    api_key=os.getenv("OPENAI_API_KEY", None),
)



from distilabel.llm import OpenAILLM
from distilabel.tasks import UltraJudgeTask

labeller = OpenAILLM(
    model="gpt-3.5-turbo",
    task=UltraJudgeTask(),
    prompt_format="openai",
    max_new_tokens=2048,
    temperature=0.0,
)

outputs = labeller.generate(
    inputs=[
        {
            "input": "Here's a math problem that you need to resolve: 2 + 2 * 3. What's the result of this problem? Explain it",
            "generations": [
                (
                    "The output of the math problem 2 + 2 * 3 is calculated by following "
                    "the order of operations (PEMDAS). First, perform the multiplication: "
                    "2 * 3 = 6. Then, perform the addition: 2 + 6 = 8. Therefore, the "
                    "output of the problem is 8."
                ),
                (
                    "The correct solution to the math problem is 8. To get the correct "
                    "answer, we follow the order of operations (PEMDAS) and perform "
                    "multiplication before addition. So, first, we solve 2 * 3 = 6, "
                    "then we add 2 to 6 to get 8."
                ),
            ],
        }
    ]
)

print(outputs[0][0]["parsed_output"])



from datasets import load_dataset
from distilabel.llm import LlamaCppLLM
from distilabel.pipeline import Pipeline
from distilabel.tasks import TextGenerationTask
from llama_cpp import Llama

dataset = load_dataset("argilla/distilabel-docs", split="train")
dataset = dataset.remove_columns(
    [column for column in dataset.column_names if column not in ["input"]]
)

pipeline = Pipeline(
    generator=LlamaCppLLM(
        model=Llama(
            model_path="./llama-2-7b-chat.Q4_0.gguf",
            verbose=False,
            n_ctx=1024,
        ),
        task=TextGenerationTask(),
        max_new_tokens=512,
        prompt_format="llama2",
    ),
)


dataset = pipeline.generate(dataset, num_generations=2, batch_size=5)



from datasets import load_dataset
from distilabel.llm import OpenAILLM
from distilabel.pipeline import pipeline
from distilabel.tasks import TextGenerationTask

dataset = (
    load_dataset("HuggingFaceH4/instruction-dataset", split="test[:10]")
    .remove_columns(["completion", "meta"])
    .rename_column("prompt", "input")
)

task = TextGenerationTask()  # (1)

generator = OpenAILLM(task=task, max_new_tokens=512)  # (2)

pipeline = pipeline("preference", "instruction-following", generator=generator)  # (3)

dataset = pipeline.generate(dataset)



# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

if sys.version_info < (3, 9):
    import importlib_resources
else:
    import importlib.resources as importlib_resources
from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any, Dict, Generator, List, Literal, Union

from jinja2 import Template

from distilabel.tasks.prompt import Prompt
from distilabel.utils.serialization import _Serializable

if TYPE_CHECKING:
    from argilla import FeedbackDataset, FeedbackRecord


def get_template(template_name: str) -> str:
    return str(
        importlib_resources.files("distilabel") / "tasks/_templates" / template_name
    )


class Task(ABC, _Serializable):
    """Abstract class used to define the methods required to create a `Task`, to be used
    within an `LLM`.

    Args:
        system_prompt (str): the system prompt to be used for generation.
        task_description (Union[str, None], optional): the description of the task. Defaults to `None`.

    Raises:
        ValueError: if the `__jinja2_template__` attribute is not provided.
    """

    system_prompt: str
    task_description: Union[str, None] = None

    __jinja2_template__: Union[str, None] = None
    __type__: Union[Literal["generation", "labelling"], None] = None

    def __rich_repr__(self) -> Generator[Any, None, None]:
        yield "system_prompt", self.system_prompt
        yield "task_description", self.task_description
        yield "input_args_names", self.input_args_names
        yield "output_args_names", self.output_args_names

    @property
    def template(self) -> "Template":
        if self.__jinja2_template__ is None:
            raise ValueError(
                "You must provide a `__jinja2_template__` attribute to your Task subclass."
            )

        return Template(open(self.__jinja2_template__).read())

    @abstractmethod
    def generate_prompt(self, **kwargs: Any) -> Prompt:
        pass

    @abstractmethod
    def parse_output(self, output: str) -> Any:
        pass

    @property
    @abstractmethod
    def input_args_names(self) -> List[str]:
        pass

    @property
    @abstractmethod
    def output_args_names(self) -> List[str]:
        pass

    def validate_dataset(self, columns_in_dataset: List[str]) -> None:
        """Validates that the dataset contains the required columns for the task.

        Args:
            columns_in_dataset (List[str]): the columns in the dataset.

        Raises:
            KeyError: if the dataset does not contain the required columns.
        """
        for input_arg_name in self.input_args_names:
            if input_arg_name not in columns_in_dataset:
                raise KeyError(
                    f"LLM expects a column named '{input_arg_name}' in the provided"
                    " dataset, but it was not found."
                )

    def to_argilla_dataset(
        self, dataset_row: Dict[str, Any], *args: Any, **kwargs: Any
    ) -> "FeedbackDataset":
        raise NotImplementedError(
            "`to_argilla_dataset` is not implemented, if you want to export your dataset as an Argilla"
            " `FeedbackDataset` you will need to implement this method first."
        )

    def to_argilla_record(
        self, dataset_row: Dict[str, Any], *args: Any, **kwargs: Any
    ) -> Union["FeedbackRecord", List["FeedbackRecord"]]:
        raise NotImplementedError(
            "`to_argilla_record` is not implemented, if you want to export your dataset as an Argilla"
            " `FeedbackDataset` you will need to implement this method first."
        )

    # Renamed to _to_argilla_record instead of renaming `to_argilla_record` to protected, as that would
    # imply more breaking changes.
    def _to_argilla_record(  # noqa: C901
        self, dataset_row: Dict[str, Any], *args: Any, **kwargs: Any
    ) -> Union["FeedbackRecord", List["FeedbackRecord"]]:
        column_names = list(dataset_row.keys())
        if self.__type__ is None or self.__type__ == "generation":
            required_column_names = self.input_args_names + self.output_args_names
        elif self.__type__ == "labelling":
            required_column_names = self.output_args_names
        else:
            raise ValueError("The task type is not supported.")

        dataset_rows = [dataset_row]
        if "generation_model" in dataset_row and isinstance(
            dataset_row["generation_model"], list
        ):
            generation_columns = column_names[
                column_names.index("generation_model") : column_names.index(
                    "labelling_model"
                )
                if "labelling_model" in column_names
                else None
            ]
            if any(
                isinstance(nested, list)
                for column_name in list(
                    set(generation_columns)
                    - {
                        "generation_model",
                        "generation_prompt",
                        "raw_generation_response",
                    }
                )
                for nested in dataset_row[column_name]
            ):
                if any(
                    generation_column in required_column_names
                    for generation_column in generation_columns
                ):
                    unwrapped_dataset_rows = []
                    for row in dataset_rows:
                        for idx in range(len(dataset_row["generation_model"])):
                            unwrapped_dataset_row = {}
                            for key, value in row.items():
                                if key in generation_columns:
                                    unwrapped_dataset_row[key] = value[idx]
                                else:
                                    unwrapped_dataset_row[key] = value
                            unwrapped_dataset_rows.append(unwrapped_dataset_row)
                    dataset_rows = unwrapped_dataset_rows

        if "labelling_model" in dataset_row and isinstance(
            dataset_row["labelling_model"], list
        ):
            labelling_columns = column_names[column_names.index("labelling_model") :]
            if any(
                isinstance(nested, list)
                for column_name in list(
                    set(labelling_columns)
                    - {
                        "labelling_model",
                        "labelling_prompt",
                        "raw_labelling_response",
                    }
                )
                for nested in dataset_row[column_name]
            ):
                if any(
                    labelling_column in required_column_names
                    for labelling_column in labelling_columns
                ):
                    unwrapped_dataset_rows = []
                    for row in dataset_rows:
                        for idx in range(len(dataset_row["labelling_model"])):
                            unwrapped_dataset_row = {}
                            for key, value in row.items():
                                if key in labelling_columns:
                                    unwrapped_dataset_row[key] = value[idx]
                                else:
                                    unwrapped_dataset_row[key] = value
                            unwrapped_dataset_rows.append(unwrapped_dataset_row)
                    dataset_rows = unwrapped_dataset_rows

        if len(dataset_rows) == 1:
            return self.to_argilla_record(dataset_rows[0], *args, **kwargs)

        records = []
        for dataset_row in dataset_rows:
            generated_records = self.to_argilla_record(dataset_row, *args, **kwargs)
            if isinstance(generated_records, list):
                records.extend(generated_records)
            else:
                records.append(generated_records)
        return records



# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# WARNING: To run this example, you will need to install `accelerate` as
# `pip install accelerate`

# Usage: `accelerate launch examples/pipeline-accelerate-and-openai.py`

import os

import torch
from accelerate import Accelerator
from accelerate.utils import gather_object
from datasets import Dataset, load_dataset
from distilabel.dataset import CustomDataset
from distilabel.llm import OpenAILLM, TransformersLLM
from distilabel.pipeline import Pipeline
from distilabel.tasks import TextGenerationTask, UltraFeedbackTask
from transformers import AutoModelForCausalLM, AutoTokenizer


def get_current_device() -> int:
    """Get the current device. For GPU we return the local process index to enable multiple GPU training."""
    return Accelerator().local_process_index if torch.cuda.is_available() else "cpu"  # type: ignore


if __name__ == "__main__":
    accelerator = Accelerator()
    with accelerator.local_main_process_first():
        dataset = (
            load_dataset("HuggingFaceH4/instruction-dataset", split="test[:10]")
            .remove_columns(["completion", "meta"])
            .rename_column("prompt", "input")
        )

    model = AutoModelForCausalLM.from_pretrained(
        "HuggingFaceH4/zephyr-7b-beta",
        torch_dtype=torch.bfloat16,
        device_map={"": get_current_device()},
    )
    tokenizer = AutoTokenizer.from_pretrained("HuggingFaceH4/zephyr-7b-beta")
    tokenizer.padding_side = "left"

    pipeline = Pipeline(
        generator=TransformersLLM(
            model=model,
            tokenizer=tokenizer,
            task=TextGenerationTask(),
            max_new_tokens=128,
            temperature=0.3,
            prompt_format="zephyr",
            do_sample=True,
        ),
        labeller=OpenAILLM(
            model="gpt-3.5-turbo",
            task=UltraFeedbackTask.for_instruction_following(),
            max_new_tokens=128,
            num_threads=2,
            api_key=os.getenv("OPENAI_API_KEY", None),
            temperature=0.0,
        ),
    )
    with accelerator.split_between_processes(dataset.to_dict()) as inputs:  # type: ignore
        inputs = Dataset.from_dict(inputs)  # type: ignore
        dataset = pipeline.generate(
            inputs,  # type: ignore
            num_generations=2,
            batch_size=1,
            checkpoint_strategy=True,
            display_progress_bar=True,
        )
        dataset = gather_object(dataset)

    # Push to the HuggingFace Hub
    if accelerator.is_main_process:
        dataset = Dataset.from_list(dataset)
        dataset.push_to_hub(
            os.getenv("HF_REPO_ID"),  # type: ignore
            split="train",
            private=True,
            token=os.getenv("HF_TOKEN", None),
        )

        try:
            from uuid import uuid4

            import argilla as rg

            rg.init(
                api_url=os.getenv("ARGILLA_API_URL"),
                api_key=os.getenv("ARGILLA_API_KEY"),
            )

            # Convert into an Argilla dataset and push it to Argilla
            dataset.__class__ = CustomDataset
            dataset.task = UltraFeedbackTask.for_instruction_following()  # type: ignore
            rg_dataset = dataset.to_argilla()  # type: ignore
            rg_dataset.push_to_argilla(
                name=f"my-dataset-{uuid4()}",
                workspace="admin",
            )
        except ImportError:
            pass

    accelerator.wait_for_everyone()



# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
import random
from collections import defaultdict
from typing import TYPE_CHECKING, Any, Dict, List, Literal, Optional, Tuple, get_args

from distilabel.logger import get_logger

if TYPE_CHECKING:
    from distilabel.dataset import CustomDataset

logger = get_logger()

BinarizationStrategies = Literal["random", "worst"]


def _get_best_response(
    example: Any,
    rating_column: str = "rating",
    responses_column: str = "generations",
    rationale_column: Optional[str] = "rationale",
) -> Tuple[str, int, str, str]:
    """Helper function to get the best response from an example, this can be used
    independent on the method to chose the rejected response.

    Also, it removes the best response from the example.

    Args:
        example (Any): Each row in the dataset as passed when calling the map function on a datasets.Dataset.
        rating_column (str, optional):
            Column containing the rating in the CustomDataset. Defaults to "rating".
        responses_column (str, optional):
            Column containing the responses from a model in a CustomDataset. Defaults to "generations".
        rationale_column (Optional[str], optional):
            Column containing the rationale from a model in a CustomDataset, if found, otherwise
            will be None. Defaults to "rationale".

    Returns:
        Tuple[str, int, str, str]: Contains the prompt, best rating, chosen response, and chosen model.
    """
    # Pick the highest rating
    prompt = example["input"]
    best_rating = max(example[rating_column])
    best_response_idx = example[rating_column].index(best_rating)
    chosen_response = example[responses_column][best_response_idx]
    chosen_model = example["generation_model"][best_response_idx]
    chosen_rationale = (
        example[rationale_column][best_response_idx] if rationale_column else None
    )

    # Remove best response
    example[rating_column].pop(best_response_idx)
    example[responses_column].pop(best_response_idx)
    example["generation_model"].pop(best_response_idx)
    if chosen_rationale:
        example[rationale_column].pop(best_response_idx)
    return prompt, best_rating, chosen_response, chosen_model, chosen_rationale


def _format_message(prompt: str, response: str) -> List[Dict[str, str]]:
    """Helper function to format the messages (chosen/rejected) in OpenAI format.

    Returns:
        message: List of dictionaries with the OpenAI format.
    """
    return [
        {"role": "user", "content": prompt},
        {"role": "assistant", "content": response},
    ]


def _binarize_dataset(
    dataset: "CustomDataset",
    seed: int = None,
    strategy: BinarizationStrategies = "random",
    keep_ties: bool = False,
    rating_column: str = "rating",
    responses_column: str = "generations",
    rationale_column: Optional[str] = "rationale",
    **kwargs: Any,
) -> "CustomDataset":
    """Binarizes a distilabel dataset.

    Args:
        dataset (CustomDataset): The distilabel dataset to binarize.
        seed (int, optional): Random seed. Defaults to 42.
        strategy (BinarizationStrategies, optional): Method to binarize the data. Defaults to "random".
        keep_ties (bool, optional):
            Whether to keep ties in case the binarization method generated the chosen
            and rejected responses to have the same rating. Defaults to False.
        kwargs: Extra parameters passed to `datasets.Dataset.map`.

    Raises:
        ValueError: If the strategy is not implemented.

    Returns:
        CustomDataset: Dataset binarized.
    """
    get_best_response = functools.partial(
        _get_best_response,
        rating_column=rating_column,
        responses_column=responses_column,
        rationale_column=rationale_column,
    )

    def binarize_random(example):
        (
            prompt,
            best_rating,
            chosen_response,
            chosen_model,
            chosen_rationale,
        ) = get_best_response(example)
        random.seed(seed)

        # Then you pick the rejected from the list of candidates with lower scores.
        example_lower = defaultdict(list)
        for i, rating in enumerate(example[rating_column]):
            if rating < best_rating:
                example_lower[responses_column].append(example[responses_column][i])
                example_lower[rating_column].append(rating)
                # Add the rationale if found
                if chosen_rationale:
                    example_lower[rationale_column].append(example[rationale_column][i])
        # Otherwise you declare that a tie
        if len(example_lower[rating_column]) == 0:
            # In this case we don't have any response with a lower rating, so we just
            # let the original example (we have a tie)
            example_lower = example

        random_response = random.choice(example_lower[responses_column])
        random_response_idx = example_lower[responses_column].index(random_response)
        random_rating = example_lower[rating_column][random_response_idx]

        random_model = example["generation_model"][random_response_idx]

        bin_example = {
            "prompt": prompt,
            "chosen": _format_message(prompt, chosen_response),
            "rejected": _format_message(prompt, random_response),
            "rating_chosen": int(best_rating),
            "rating_rejected": int(random_rating),
            "chosen_model": chosen_model,
            "rejected_model": random_model,
        }
        if chosen_rationale:
            bin_example["chosen_rationale"] = chosen_rationale
            bin_example["rejected_rationale"] = example[rationale_column][
                random_response_idx
            ]

        return bin_example

    def binarize_worst(example):
        (
            prompt,
            best_rating,
            chosen_response,
            chosen_model,
            chosen_rationale,
        ) = get_best_response(example)

        worst_rating = min(example[rating_column])
        worst_response_idx = example[rating_column].index(worst_rating)
        worst_response = example[responses_column][worst_response_idx]
        worst_model = example["generation_model"][worst_response_idx]

        bin_example = {
            "prompt": prompt,
            "chosen": _format_message(prompt, chosen_response),
            "rejected": _format_message(prompt, worst_response),
            "rating_chosen": int(best_rating),
            "rating_rejected": int(worst_rating),
            "chosen_model": chosen_model,
            "rejected_model": worst_model,
        }
        if chosen_rationale:
            bin_example["chosen_rationale"] = chosen_rationale
            bin_example["rejected_rationale"] = example[rationale_column][
                worst_response_idx
            ]

        return bin_example

    if strategy == "random":
        binarization_method = binarize_random
    elif strategy == "worst":
        binarization_method = binarize_worst
    else:
        raise ValueError(
            f"Strategy `{strategy}` is not implemented, it must be one of: {get_args(BinarizationStrategies)}"
        )

    if "generation_model" not in dataset.column_names:
        # Ensure generation model is found in the dataset, even if empty, to avoid
        # erros when calling map
        dataset = dataset.add_column(
            "generation_model", [[""] * len(dataset[0]["generations"])] * len(dataset)
        )

    dataset = dataset.map(binarization_method, **kwargs)

    if not keep_ties:
        dataset = dataset.filter(
            lambda example: example["rating_chosen"] != example["rating_rejected"]
        )
    return dataset


def prepare_dataset(
    dataset: "CustomDataset",
    strategy: BinarizationStrategies = "random",
    sft: bool = False,
    seed: Optional[int] = None,
    keep_ties: bool = False,
    **kwargs: Any,
) -> "CustomDataset":
    """Helper function to prepare a distilabel dataset for training with the standard formats.

    Currently supports the `PreferenceTask`, and binarizes the responses assuming
    one of two strategies:

    - `random`: Selects the *chosen* response based on the highest rating, and for the
        *rejected* selects a random response from the remaining ones. Filters the examples in which
        the chosen rating is equal to the rejected one.
    - `worst`: Selects the *chosen* response based on the highest rating, and for the
        *rejected* selects the response with the lowest rating. Filters the examples in which the
        chosen rating is equal to the rejected one.

    Take a look at [argilla/ultrafeedback-binarized-preferences](https://huggingface.co/datasets/argilla/ultrafeedback-binarized-preferences)
    for more information on binarizing a dataset to prepare it for DPO fine-tuning.

    Expected format for a dataset to be trained with DPO as defined in trl's
    [dpo trainer](https://huggingface.co/docs/trl/main/en/dpo_trainer#expected-dataset-format).

    Note:
        Take a look at the
        [Prepare datasets for fine-tuning](https://distilabel.argilla.io/latest/technical-reference/pipeline/#prepare-datasets-for-fine-tuning)
        section in the Concept guides for more information on the binarization process.

    Args:
        dataset (CustomDataset):
            CustomDataset with a PreferenceTask to prepare for Direct Preference Optimization.
        strategy (BinarizationStrategies, optional):
            Strategy to binarize the data. Defaults to "random".
        sft (bool, optional):
            Whether to add a `messages` column to the dataset, to be used for Supervised Fine Tuning.
            If set to True, this messages column will contain the same information as the chosen response.
            Defaults to False.
        seed (int, optional): Seed for the random generator, in case of `random` strategy. Defaults to None.
        keep_ties (bool, optional):
            Whether to keep ties in case the binarization method generated the chosen
            and rejected responses to have the same rating. Defaults to False.
        kwargs: Extra parameters passed to `datasets.Dataset.map`.

    Returns:
        CustomDataset: Dataset formatted for training with DPO.

    Examples:
        >>> from datasets import load_dataset
        >>> from distilabel.tasks import UltraFeedbackTask
        >>> import os
        >>> dataset = load_dataset("argilla/DistiCoder-dpo", token=os.getenv("HF_API_TOKEN"), split="train")
        >>> dataset.task = UltraFeedbackTask.for_instruction_following()
        >>> dataset_binarized = prepare_dataset(dataset, strategy="worst")
    """
    from distilabel.tasks.preference.base import PreferenceTask

    if not isinstance(dataset.task, PreferenceTask):
        raise ValueError(
            "This functionality is currently implemented for `PreferenceTask` only."
        )

    remove_columns = [
        "input",
        "generation_model",
        "generations",
        "rating",
        "labelling_model",
        "labelling_prompt",
        "raw_labelling_response",
    ]

    # Remove the rows for which there is no rating
    def remove_incomplete_rows(example):
        if not example["rating"]:
            return False
        if len(example["generations"]) != len(example["rating"]):
            return False
        # TODO(plaguss): Maybe we should remove the examples with less than 2 generations
        # instead of checking after the filtering
        return True

    initial_length = len(dataset)
    dataset = dataset.filter(remove_incomplete_rows)
    if len(dataset) != initial_length:
        logger.info(
            f"Found {initial_length - len(dataset)} examples with no rating or different number of ratings than generations, removing them."
        )

    if len(dataset[0]["generations"]) < 2:
        raise ValueError("The dataset must contain at least 2 generations per example.")

    # If the dataset contains the rationale, grab the content
    if "rationale" in dataset.column_names:
        rationale_column = "rationale"
        remove_columns.append("rationale")
    else:
        rationale_column = None

    ds = _binarize_dataset(
        dataset,
        strategy=strategy,
        seed=seed,
        keep_ties=keep_ties,
        rating_column="rating",
        responses_column="generations",
        rationale_column=rationale_column,
        **kwargs,
    )

    if sft:
        # Adds a column to be used for Supervised Fine Tuning based on the chosen response
        ds = ds.map(
            lambda example: {
                **example,
                "messages": example["chosen"],
            }
        )
    # Imported here to avoid circular imports
    from distilabel.dataset import CustomDataset

    ds = ds.remove_columns(remove_columns)
    ds.__class__ = CustomDataset
    ds.task = dataset.task
    return ds



# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest
from distilabel.tasks.text_generation.base import TextGenerationTask
from distilabel.tasks.text_generation.principles import UltraFeedbackPrinciples


class TestSuiteTextGenerationTask:
    def test_init(self) -> None:
        task = TextGenerationTask()
        assert task.system_prompt == (
            "You are a helpful, respectful and honest assistant. Always answer as helpfully as possible,"
            " while being safe. Your answers should not include any harmful, unethical, racist, sexist,"
            " toxic, dangerous, or illegal content. Please ensure that your responses are socially unbiased"
            " and positive in nature.\nIf a question does not make any sense, or is not factually coherent,"
            " explain why instead of answering something not correct. If you don't know the answer to a"
            " question, please don't share false information."
        )
        assert task.principles == {
            "harmlessness": UltraFeedbackPrinciples.harmlessness,
            "helpfulness": UltraFeedbackPrinciples.helpfulness,
            "truthfulness": UltraFeedbackPrinciples.truthfulness,
            "honesty": UltraFeedbackPrinciples.honesty,
            "verbalized_calibration": UltraFeedbackPrinciples.verbalized_calibration,
        }
        assert task.principles_distribution is None

    def test_init_with_missing_principles_distribution(self) -> None:
        with pytest.raises(
            ValueError,
            match="Principles 'helpfulness', 'truthfulness', 'honesty', 'verbalized_calibration' included in `principles` is not in `principles_distribution`",
        ):
            TextGenerationTask(principles_distribution={"harmlessness": 1.0})

    def test_init_with_principles_distribution_not_suming_up_1(self) -> None:
        with pytest.raises(
            ValueError, match="`principles_distribution` must sum to 1.0"
        ):
            TextGenerationTask(
                principles_distribution={
                    "harmlessness": 1.0,
                    "helpfulness": 1.0,
                    "truthfulness": 1.0,
                    "honesty": 1.0,
                    "verbalized_calibration": 1.0,
                }
            )

    def test_generate_prompt(self) -> None:
        task = TextGenerationTask(system_prompt="This is a custom system prompt")
        prompt = task.generate_prompt(input="Generate something my boy")
        assert prompt.system_prompt == "This is a custom system prompt"
        assert prompt.formatted_prompt == "Generate something my boy"

    def test_generate_prompt_with_default(self) -> None:
        task = TextGenerationTask()
        prompt = task.generate_prompt(input="Generate something my boy")
        assert prompt.system_prompt == task.system_prompt
        assert prompt.formatted_prompt == "Generate something my boy"

    def test_generate_prompt_with_balanced_principles(self) -> None:
        task = TextGenerationTask(
            principles={"helpfulness": ["Help! I need somebody..."]},
            principles_distribution="balanced",
        )
        prompt = task.generate_prompt(input="Generate something my boy")
        assert (
            prompt.system_prompt
            == task.system_prompt + " " + "Help! I need somebody..."
        )

    def test_generate_prompt_with_distribution_principles(self) -> None:
        task = TextGenerationTask(
            principles={
                "helpfulness": ["Help! I need somebody..."],
                "honesty": ["I'm honest, I swear!"],
            },
            principles_distribution={"helpfulness": 1.0, "honesty": 0.0},
        )
        prompt = task.generate_prompt(input="Generate something my boy")
        assert (
            prompt.system_prompt
            == task.system_prompt + " " + "Help! I need somebody..."
        )



# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argilla as rg
import pytest
from distilabel.dataset import CustomDataset
from distilabel.tasks.text_generation.self_instruct import (
    SelfInstructTask,
)

breadth = """I want you to act as a Prompt Creator.
Your goal is to draw inspiration from the #Given Prompt# to create a brand new prompt.
This new prompt should belong to the same domain as the #Given Prompt# but must be even more rare.
The LENGTH and complexity of the #Created Prompt# should be similar to that of the #Given Prompt#.
The #Created Prompt# must be reasonable and must be understood and responded by humans.
'#Given Prompt#', '#Created Prompt#', 'given prompt' and 'created prompt' are not allowed to appear in #Created Prompt#
#Given Prompt#:
HELLO

#Created Prompt#:
"""

default_prompt = """# Task Description
Develop 5 user queries that can be received by the given AI application and applicable to the provided context. Emphasize diversity in verbs and linguistic structures within the model's textual capabilities.

# Criteria for Queries
Incorporate a diverse range of verbs, avoiding repetition.
Ensure queries are compatible with AI model's text generation functions and are limited to 1-2 sentences.
Design queries to be self-contained and standalone.
Blend interrogative (e.g., "What is the significance of x?") and imperative (e.g., "Detail the process of x.") styles.
Write each query on a separate line and avoid using numbered lists or bullet points.

# AI Application
AI assistant

# Context
HELLO

# Output
"""

custom_prompt = """# Task Description
Develop 5 user queries that can be received by the given AI application and applicable to the provided context. Emphasize diversity in verbs and linguistic structures within the model's textual capabilities.

# Criteria for Queries
Incorporate a diverse range of verbs, avoiding repetition.
Ensure queries are compatible with AI model's text generation functions and are limited to 1-2 sentences.
Design queries to be self-contained and standalone.
Write each query on a separate line and avoid using numbered lists or bullet points.

# AI Application
An AI assistant adept at writing Haiku.
It expects complete suggestions from users providing details of the kind of haiku they want.
The AI assistant will help users write haiku about particular topics and is willing to accept requests related to a specific subject or object or a more abstract request
based on an emotion, theme or vibe.

# Context
HELLO

# Output
"""

application_description = """An AI assistant adept at writing Haiku.
It expects complete suggestions from users providing details of the kind of haiku they want.
The AI assistant will help users write haiku about particular topics and is willing to accept requests related to a specific subject or object or a more abstract request
based on an emotion, theme or vibe."""

criteria_for_query_generation = """Incorporate a diverse range of verbs, avoiding repetition.
Ensure queries are compatible with AI model's text generation functions and are limited to 1-2 sentences.
Design queries to be self-contained and standalone."""


@pytest.mark.parametrize(
    "self_instruct, expected",
    [
        (SelfInstructTask(), default_prompt),
        (
            SelfInstructTask(
                application_description=application_description,
                criteria_for_query_generation=criteria_for_query_generation,
            ),
            custom_prompt,
        ),
    ],
)
def test_self_instruct_task(self_instruct: SelfInstructTask, expected: str):
    assert isinstance(self_instruct, SelfInstructTask)
    assert (
        self_instruct.system_prompt
        == "You are an expert prompt writer, writing the best and most diverse prompts for a variety of tasks. You are given a task description and a set of instructions for how to write the prompts for an specific AI application."
    )
    assert self_instruct.generate_prompt(input="HELLO").formatted_prompt == expected


@pytest.fixture
def custom_self_instruct_dataset() -> CustomDataset:
    ds = CustomDataset.from_dict(
        {
            "input": [
                "EN EN\nEUROPEAN\nCOMMISSION\nProposal for a\nREGULATION OF THE EUROPEAN PARLIAMENT AND OF THE COUNCIL\nLAYING DOWN HARMONISED RULES ON ARTIFICIAL INTELLIGENCE\n(ARTIFICIAL INTELLIGENCE ACT) AND AMENDING CERTAIN UNION\nLEGISLATIVE ACTS\x0cEN\nEXPLANATORY MEMORANDUM\n1. CONTEXT OF THE PROPOSAL\n1.1. Reasons for and objectives of the proposal\nThis explanatory memorandum accompanies the proposal for a Regulation laying down\nharmonised rules on artificial intelligence (Artificial Intelligence Act). Artificial Intelligence\n(AI) is a fast evolving family of technologies that can bring a wide array of economic and\nsocietal benefits across the entire spectrum of industries and social activities. By improving\nprediction, optimising operations and resource allocation, and personalising service delivery,\nthe use of artificial intelligence can support socially and environmentally beneficial outcomes\nand provide key competitive advantages to companies and the European economy. ",
                "EN EN\nEUROPEAN\nCOMMISSION\nProposal for a\nREGULATION OF THE EUROPEAN PARLIAMENT AND OF THE COUNCIL\nLAYING DOWN HARMONISED RULES ON ARTIFICIAL INTELLIGENCE\n(ARTIFICIAL INTELLIGENCE ACT) AND AMENDING CERTAIN UNION\nLEGISLATIVE ACTS\x0cEN\nEXPLANATORY MEMORANDUM\n1. CONTEXT OF THE PROPOSAL\n1.1. Reasons for and objectives of the proposal\nThis explanatory memorandum accompanies the proposal for a Regulation laying down\nharmonised rules on artificial intelligence (Artificial Intelligence Act). Artificial Intelligence\n(AI) is a fast evolving family of technologies that can bring a wide array of economic and\nsocietal benefits across the entire spectrum of industries and social activities. By improving\nprediction, optimising operations and resource allocation, and personalising service delivery,\nthe use of artificial intelligence can support socially and environmentally beneficial outcomes\nand provide key competitive advantages to companies and the European economy. ",
            ],
            "generation_model": [["argilla/notus-7b-v1"], ["argilla/notus-7b-v1"]],
            "generation_prompt": [
                [
                    'You are an expert prompt writer, writing the best and most diverse prompts for a variety of tasks. You are given a task description and a set of instructions for how to write the prompts for an specific AI application.\n# Task Description\nDevelop 5 user queries that can be received by the given AI application and applicable to the provided context. Emphasize diversity in verbs and linguistic structures within the model\'s textual capabilities.\n\n# Criteria for Queries\nIncorporate a diverse range of verbs, avoiding repetition.\nEnsure queries are compatible with AI model\'s text generation functions and are limited to 1-2 sentences.\nDesign queries to be self-contained and standalone.\nBlend interrogative (e.g., "What is the significance of x?") and imperative (e.g., "Detail the process of x.") styles.\nWrite each query on a separate line and avoid using numbered lists or bullet points.\n\n# AI Application\nA assistant that can answer questions about the AI Act made by the European Union.\n\n# Context\nEN EN\nEUROPEAN\nCOMMISSION\nProposal for a\nREGULATION OF THE EUROPEAN PARLIAMENT AND OF THE COUNCIL\nLAYING DOWN HARMONISED RULES ON ARTIFICIAL INTELLIGENCE\n(ARTIFICIAL INTELLIGENCE ACT) AND AMENDING CERTAIN UNION\nLEGISLATIVE ACTS\x0cEN\nEXPLANATORY MEMORANDUM\n1. CONTEXT OF THE PROPOSAL\n1.1. Reasons for and objectives of the proposal\nThis explanatory memorandum accompanies the proposal for a Regulation laying down\nharmonised rules on artificial intelligence (Artificial Intelligence Act). Artificial Intelligence\n(AI) is a fast evolving family of technologies that can bring a wide array of economic and\nsocietal benefits across the entire spectrum of industries and social activities. By improving\nprediction, optimising operations and resource allocation, and personalising service delivery,\nthe use of artificial intelligence can support socially and environmentally beneficial outcomes\nand provide key competitive advantages to companies and the European economy. \n\n# Output\n'
                ],
                [
                    'You are an expert prompt writer, writing the best and most diverse prompts for a variety of tasks. You are given a task description and a set of instructions for how to write the prompts for an specific AI application.\n# Task Description\nDevelop 5 user queries that can be received by the given AI application and applicable to the provided context. Emphasize diversity in verbs and linguistic structures within the model\'s textual capabilities.\n\n# Criteria for Queries\nIncorporate a diverse range of verbs, avoiding repetition.\nEnsure queries are compatible with AI model\'s text generation functions and are limited to 1-2 sentences.\nDesign queries to be self-contained and standalone.\nBlend interrogative (e.g., "What is the significance of x?") and imperative (e.g., "Detail the process of x.") styles.\nWrite each query on a separate line and avoid using numbered lists or bullet points.\n\n# AI Application\nA assistant that can answer questions about the AI Act made by the European Union.\n\n# Context\nEN EN\nEUROPEAN\nCOMMISSION\nProposal for a\nREGULATION OF THE EUROPEAN PARLIAMENT AND OF THE COUNCIL\nLAYING DOWN HARMONISED RULES ON ARTIFICIAL INTELLIGENCE\n(ARTIFICIAL INTELLIGENCE ACT) AND AMENDING CERTAIN UNION\nLEGISLATIVE ACTS\x0cEN\nEXPLANATORY MEMORANDUM\n1. CONTEXT OF THE PROPOSAL\n1.1. Reasons for and objectives of the proposal\nThis explanatory memorandum accompanies the proposal for a Regulation laying down\nharmonised rules on artificial intelligence (Artificial Intelligence Act). Artificial Intelligence\n(AI) is a fast evolving family of technologies that can bring a wide array of economic and\nsocietal benefits across the entire spectrum of industries and social activities. By improving\nprediction, optimising operations and resource allocation, and personalising service delivery,\nthe use of artificial intelligence can support socially and environmentally beneficial outcomes\nand provide key competitive advantages to companies and the European economy. \n\n# Output\n'
                ],
            ],
            "raw_generation_responses": [
                [
                    "1. What are the reasons for and objectives of the proposal for a Regulation laying down harmonised rules on artificial intelligence?\n2. How can artificial intelligence improve prediction, optimise operations and resource allocation, and personalise service delivery?\n3. What benefits can artificial intelligence bring to the European economy and society as a whole?\n4. How can the use of artificial intelligence support socially and environmentally beneficial outcomes?\n5. What competitive advantages can companies gain from using artificial intelligence?"
                ],
                [
                    "1. What are the reasons for and objectives of the proposal for a Regulation laying down harmonised rules on artificial intelligence?\n2. How can artificial intelligence improve prediction, optimise operations and resource allocation, and personalise service delivery?\n3. What benefits can artificial intelligence bring to the European economy and society as a whole?\n4. How can the use of artificial intelligence support socially and environmentally beneficial outcomes?\n5. What competitive advantages can companies gain from using artificial intelligence?"
                ],
            ],
            "instructions": [
                [
                    "What are the reasons for and objectives of the proposal for a Regulation laying down harmonised rules on artificial intelligence?",
                    "How can artificial intelligence improve prediction, optimise operations and resource allocation, and personalise service delivery?",
                    "What benefits can artificial intelligence bring to the European economy and society as a whole?",
                    "How can the use of artificial intelligence support socially and environmentally beneficial outcomes?",
                ],
                [
                    "What are the reasons for and objectives of the proposal for a Regulation laying down harmonised rules on artificial intelligence?",
                    "How can artificial intelligence improve prediction, optimise operations and resource allocation, and personalise service delivery?",
                    "What benefits can artificial intelligence bring to the European economy and society as a whole?",
                    "How can the use of artificial intelligence support socially and environmentally beneficial outcomes?",
                ],
            ],
        }
    )
    ds.task = SelfInstructTask()
    return ds


def test_self_instruct_task_to_argilla_dataset(custom_self_instruct_dataset):
    ds_row = custom_self_instruct_dataset[0]
    task = custom_self_instruct_dataset.task
    rg_dataset = task.to_argilla_dataset(ds_row)
    assert isinstance(rg_dataset, rg.FeedbackDataset)
    assert len(rg_dataset.records) == 0


def test_self_instruct_task_to_argilla_record(custom_self_instruct_dataset):
    ds_row = custom_self_instruct_dataset[0]
    task = custom_self_instruct_dataset.task
    records = task.to_argilla_record(ds_row)
    assert isinstance(records, list)
    assert len(records) == 4
    assert isinstance(records[0], rg.FeedbackRecord)


def test_self_instruct_task_to_argilla(custom_self_instruct_dataset):
    rg_dataset = custom_self_instruct_dataset.to_argilla(
        vector_strategy=False, metric_strategy=False
    )
    assert isinstance(rg_dataset, rg.FeedbackDataset)
    assert len(rg_dataset.records) == 8



# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

from concurrent.futures import Future
from typing import Any, Union

from typing_extensions import TypeGuard, TypeVar

T = TypeVar("T")


def is_future(obj: Union[Future[T], Any]) -> TypeGuard[Future[T]]:
    """Checks if an object is a future narrowing the type.

    Args:
        obj (Future[T]): Object to check

    Returns:
        TypeGuard[Future[T]]: True if it is a future
    """
    return isinstance(obj, Future)



# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import torch
from datasets import Dataset
from distilabel.llm import TransformersLLM
from distilabel.pipeline import Pipeline
from distilabel.tasks import PrometheusTask
from transformers import AutoTokenizer, LlamaForCausalLM

if __name__ == "__main__":
    model = LlamaForCausalLM.from_pretrained(
        "kaist-ai/Prometheus-7b-v1.0", torch_dtype=torch.float16, device_map="auto"
    )
    tokenizer = AutoTokenizer.from_pretrained(
        "meta-llama/Llama-2-7b-chat-hf", token=os.getenv("HF_TOKEN")
    )
    pipeline = Pipeline(
        labeller=TransformersLLM(
            model=model,  # type: ignore
            tokenizer=tokenizer,
            task=PrometheusTask(
                scoring_criteria="Is the provided completion accurate based on the given instruction?",
                score_descriptions={
                    0: "Totaly off-topic and inaccurate",
                    1: "Incorrect and inaccurate",
                    2: "Almost correct, but partially inaccurate",
                    3: "Correct but badly phrased",
                    4: "Correct and accurate",
                },
            ),
            temperature=1.0,
            top_p=1.0,
            max_new_tokens=512,
        ),
    )

    dataset = Dataset.from_dict(
        {
            "input": ["What's the capital of Spain?"],
            "generations": ["Paris"],
            "ref_completion": ["Madrid"],
        }
    )

    dataset = pipeline.generate(
        dataset,  # type: ignore
        display_progress_bar=True,
        skip_dry_run=True,
    )



from distilabel.llm import OllamaLLM
from distilabel.tasks import TextGenerationTask

llm = OllamaLLM(
    model="notus",  # should be deployed via `ollama notus:7b-v1-q5_K_M`
    task=TextGenerationTask(),
    prompt_format="openai",
)
result = llm.generate([{"input": "What's a large language model?"}])
# >>> print(result[0][0]["parsed_output"]["generations"])
# A large language model is a type of artificial intelligence (AI) system that has been trained
# on a vast amount of text data to generate human-like language. These models are capable of
# understanding and generating complex sentences, and can be used for tasks such as language
# translation, text summarization, and natural language generation. They are typically very ...



from datasets import load_dataset

instruction_dataset = (
    load_dataset("HuggingFaceH4/instruction-dataset", split="test[:3]")
    .remove_columns(["completion", "meta"])
    .rename_column("prompt", "input")
)

pipe_dataset = pipe.generate(
    instruction_dataset,
    num_generations=2,
    batch_size=1,
    checkpoint_strategy=True,
    display_progress_bar=True,
)



# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from datasets import Dataset
from distilabel.llm import OpenAILLM
from distilabel.pipeline import Pipeline
from distilabel.tasks import SelfInstructTask

math_topics = [
    "Algebraic Expressions",
    "Linear Equations",
    "Quadratic Equations",
    "Polynomial Functions",
    "Rational Expressions",
    "Exponential Functions",
    "Logarithmic Functions",
    "Sequences and Series",
    "Matrices",
    "Determinants",
    "Complex Numbers",
    "Trigonometry",
    "Geometry",
    "Coordinate Geometry",
    "Vector Algebra",
    "Statistics",
    "Probability",
    "Calculus",
    "Differential Calculus",
    "Integral Calculus",
    "Limits and Continuity",
    "Differentiation",
    "Integration",
    "Theorems of Calculus",
    "Mathematical Reasoning",
    "Set Theory",
    "Number Theory",
    "Permutations and Combinations",
    "Binomial Theorem",
    "Arithmetic Progressions",
    "Geometric Progressions",
    "Harmonic Progressions",
    "Trigonometric Ratios",
    "Trigonometric Identities",
    "Inverse Trigonometric Functions",
    "Hyperbolic Functions",
    "Conic Sections",
    "Circle Geometry",
    "Ellipse Geometry",
    "Parabola Geometry",
    "Hyperbola Geometry",
    "Function Theory",
    "Graph Theory",
    "Differential Equations",
    "Mathematical Induction",
    "Discrete Mathematics",
    "Linear Programming",
    "Analytical Geometry",
    "Euclidean Geometry",
    "Non-Euclidean Geometry",
]


if __name__ == "__main__":
    dataset = Dataset.from_dict({"input": math_topics})

    instruction_task = SelfInstructTask(
        application_description="A question-answering assistant for engaging and challenging math quizzes and problems"
    )

    instruction_generator = OpenAILLM(
        task=instruction_task,
        api_key=os.getenv("OPENAI_API_KEY", None),
        num_threads=4,
        max_new_tokens=1024,
    )

    pipeline = Pipeline(generator=instruction_generator)

    distiset = pipeline.generate(dataset=dataset, num_generations=4, batch_size=2)

    instructions = []
    for generations in distiset["generations"]:
        for generation in generations:
            instructions.extend(generation)
    print(f"Number of generated instructions: {len(instructions)}")
    print(instructions)

    # Output:
    # Number of generated instructions: 2044
    # 1. Provide an explanation for solving a quadratic equation step by step.
    # 2. What is the process for simplifying an algebraic expression with exponents?
    # 3. Detail how to factorize a polynomial equation.
    # 4. How can one determine the maximum or minimum value of a quadratic function?
    # 5. Explain the concept of inequalities and how to solve them algebraically.
    # 6. Describe the procedure for finding the roots of a cubic equation.
    # 7. What are the different types of factoring techniques used in algebra?
    # 8. Can you outline the steps for evaluating an algebraic expression using substitution?
    # 9. Compare and contrast linear and quadratic equations in terms of their solutions and graphs.
    # 10. How can one determine if a given graph represents a linear or quadratic equation?
    # 1. How can I simplify the algebraic expression (x^2 + 3x + 2)(2x - 1)?
    # 2. Provide step-by-step instructions on how to solve the equation 4(x + 2) - 3 = 7(2x - 1).
    # 3. What is the value of x in the equation 3(x - 4) = 5x + 6?
    # 4. Detail the process of factoring the expression 12x^2 - 7x - 10.
    # 5. What is the result of expanding the binomial (2x - 3)^2?



# Copyright 2023-present, Argilla, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
from dataclasses import dataclass
from typing import Any, Dict, List

from distilabel.tasks.base import get_template
from distilabel.tasks.prompt import Prompt
from distilabel.tasks.text_generation.base import TextGenerationTask
from distilabel.tasks.text_generation.mixins import InstructTaskMixin

_SELF_INSTRUCT_TEMPLATE = get_template("self-instruct.jinja2")


@dataclass
class SelfInstructTask(InstructTaskMixin, TextGenerationTask):
    """A `TextGenerationTask` following the Self-Instruct specification for building
    the prompts.

    Args:
        system_prompt (str, optional): the system prompt to be used. Defaults to `None`.
        principles (Dict[str, List[str]], optional): the principles to be used for the system prompt.
            Defaults to `None`.
        principles_distribution (Union[Dict[str, float], Literal["balanced"], None], optional): the
            distribution of principles to be used for the system prompt. Defaults to `None`.
        application_description (str, optional): the description of the AI application. Defaults to
            "AI assistant".
        num_instructions (int, optional): the number of instructions to be used for the prompt.
            Defaults to 5.
        criteria_for_query_generation (str, optional): the criteria for query generation that we want
            our model to have. Default value covers default behaviour for SelfInstructTask. This value is
            passed to the .jinja template, where extra instructions are added to ensure correct output format.

    References:
        - [`Self-Instruct: Aligning Language Models with Self-Generated Instructions`](https://arxiv.org/abs/2212.10560)
        - [`Self-Instruct - GitHub Repository`](https://github.com/yizhongw/self-instruct)
    """

    system_prompt: str = (
        "You are an expert prompt writer, writing the best and most diverse prompts for a variety of tasks."
        " You are given a task description and a set of instructions for how to write the prompts for an"
        " specific AI application."
    )

    application_description: str = "AI assistant"
    num_instructions: int = 5

    criteria_for_query_generation: str = (
        "Incorporate a diverse range of verbs, avoiding repetition.\n"
        "Ensure queries are compatible with AI model's text generation functions and are limited to 1-2 sentences.\n"
        "Design queries to be self-contained and standalone.\n"
        'Blend interrogative (e.g., "What is the significance of x?") and imperative (e.g., "Detail the process of x.") styles.'
    )

    __jinja2_template__: str = _SELF_INSTRUCT_TEMPLATE

    def generate_prompt(self, input: str, **_: Any) -> Prompt:
        """Generates a prompt following the Self-Instruct specification.

        Args:
            input (str): the input to be used for the prompt.

        Returns:
            Prompt: the generated prompt.

        Examples:
            >>> from distilabel.tasks.text_generation import SelfInstructTask
            >>> task = SelfInstructTask(system_prompt="You are a helpful assistant.", num_instructions=2)
            >>> task.generate_prompt("What are the first 5 Fibonacci numbers?")
            Prompt(
                system_prompt="You are a helpful assistant.",
                formatted_prompt="# Task Description ...",
            )
        """
        render_kwargs = {
            "application_description": self.application_description,
            "num_instructions": self.num_instructions,
            "criteria_for_query_generation": self.criteria_for_query_generation,
            "input": input,
        }
        return Prompt(
            system_prompt=self.system_prompt,
            formatted_prompt=self.template.render(**render_kwargs),
        )

    @property
    def output_args_names(self) -> List[str]:
        return ["instructions"]

    def parse_output(self, output: str) -> Dict[str, List[str]]:
        """Parses the output of the model into the desired format."""
        pattern = re.compile(r"\d+\.\s*(.*?)\n")
        return {"instructions": pattern.findall(output)}



import os

from distilabel.llm import OpenAILLM
from distilabel.tasks import SelfInstructTask

generator = OpenAILLM(
    task=SelfInstructTask(
        system_prompt="You are a question-answering assistant for...",
        application_description="AI assistant",
        num_instructions=3,
        criteria_for_query_generation="Design queries to be... ",
    ),
    api_key=os.getenv("OPENAI_API_KEY", None),
)
